/**
 * @file
 * Dialog API inspired by HTML5 dialog element.
 *
 * @see http://www.whatwg.org/specs/web-apps/current-work/multipage/commands.html#the-dialog-element
 */

(function (Drupal) {
  /**
   * Creates a native HTML dialog element with specified content and settings.
   *
   * @param {string|HTMLElement} content - The content to be displayed in the dialog.
   * @param {Object} settings - Configuration settings for the dialog.
   * @param {string} settings.dialogClass - CSS classes to be added to the dialog.
   * @param {number|string} settings.width - Width of the dialog (in pixels or other CSS units).
   * @param {number|string} settings.height - Height of the dialog (in pixels or other CSS units).
   * @param {number|string} settings.minWidth - Minimum width of the dialog.
   * @param {number|string} settings.maxWidth - Maximum width of the dialog.
   * @param {number|string} settings.minHeight - Minimum height of the dialog.
   * @param {number|string} settings.maxHeight - Maximum height of the dialog.
   * @param {string} settings.resizable - Specify if the dialog is resizable.
   * @param {string} settings.position - Positioning of the dialog.
   * @param {string} settings.draggable - Specify if the dialog is draggable.
   * @param {boolean} settings.closeOnEscape - Whether to close the dialog on pressing the "Escape" key.
   * @param {Object} settings.classes - Additional CSS classes to be added to the dialog.
   * @param {string} settings.title - Title to be displayed in the dialog.
   * @param {Array<Object>} settings.buttons - Array of buttons to be added to the dialog.
   * @param {string} settings.buttons[].text - Text content of the button.
   * @param {string} settings.buttons[].class - CSS classes to be added to the button.
   * @param {function} settings.buttons[].click - Click event handler for the button.
   * @param {function} settings.close - Event handler for the dialog's close event.
   * @return {HTMLElement} - The created dialog container element.
   */
  function createDialog(content, settings) {
    const dialogContainer = document.createElement('dialog');
    // Add dialog classes.
    if (settings.dialogClass) {
      dialogContainer.classList.add(...settings.dialogClass.split(' '));
    }
    if (settings.width) {
      if (Number.isInteger(settings.width)) {
        dialogContainer.style.setProperty('width', `${settings.width}px`);
      } else {
        dialogContainer.style.setProperty('width', settings.width);
      }
    }
    if (settings.height) {
      if (Number.isInteger(settings.height)) {
        dialogContainer.style.setProperty('height', `${settings.height}px`);
      } else {
        dialogContainer.style.setProperty('height', settings.height);
      }
    }
    if (settings.minWidth) {
      if (Number.isInteger(settings.minWidth)) {
        dialogContainer.style.setProperty(
          'min-width',
          `${settings.minWidth}px`,
        );
      } else {
        dialogContainer.style.setProperty('min-width', settings.minWidth);
      }
    }
    if (settings.maxWidth) {
      if (Number.isInteger(settings.maxWidth)) {
        dialogContainer.style.setProperty(
          'max-width',
          `${settings.maxWidth}px`,
        );
      } else {
        dialogContainer.style.setProperty('max-width', settings.maxWidth);
      }
    }
    if (settings.minHeight) {
      if (Number.isInteger(settings.minHeight)) {
        dialogContainer.style.setProperty(
          'min-height',
          `${settings.minHeight}px`,
        );
      } else {
        dialogContainer.style.setProperty('min-height', settings.minHeight);
      }
    }
    if (settings.maxHeight) {
      if (Number.isInteger(settings.maxHeight)) {
        dialogContainer.style.setProperty(
          'max-height',
          `${settings.maxHeight}px`,
        );
      } else {
        dialogContainer.style.setProperty('max-height', settings.maxHeight);
      }
    }
    if (settings.resizable) {
      dialogContainer.style.setProperty('resize', settings.resizable);
    }
    if (settings.position) {
      dialogContainer.style.setProperty('position', settings.resizable);
    }
    if (settings.draggable) {
      dialogContainer.setAttribute('draggable', settings.draggable);
    }
    if (settings.closeOnEscape === false) {
      dialogContainer.addEventListener('cancel', (event) => {
        event.preventDefault();
      });
    }
    if (settings.classes) {
      Object.entries(settings.classes).forEach(([key, value]) => {
        dialogContainer.classList.add(key, value);
      });
    }

    // Add dialog title.
    if (settings.title) {
      dialogContainer.insertAdjacentHTML(
        'beforeend',
        `<h4>${settings.title}</h4>`,
      );
    }

    // Add dialog content.
    if (typeof content === 'string') {
      dialogContainer.insertAdjacentHTML('beforeend', content);
    } else {
      // See drupal.ajax.js line 22, where an unnecessary "drupal-modal" wrapper
      // is created around our content. Remove it here:
      if (content.id === 'drupal-modal') {
        content = content.querySelector(':first-child');
      }
      dialogContainer.appendChild(content);
    }

    // Add dialog buttons.
    if (settings.buttons) {
      settings.buttons.forEach((button) => {
        const buttonElement = document.createElement('button');
        buttonElement.textContent = button.text;
        buttonElement.classList.add(...button.class.split(' '));
        if (button.click) {
          buttonElement.addEventListener('click', button.click);
        }
        dialogContainer.appendChild(buttonElement);
      });
    }

    // Add dialog event listener and attach to body.
    dialogContainer.addEventListener('close', settings.close);
    document.body.appendChild(dialogContainer);

    return dialogContainer;
  }

  /**
   * @typedef {object} Drupal.dialog~dialogDefinition
   *
   * @param {HTMLElement|string} element
   *   The HTML element / string to be inserted into the dialog container.
   * @param {object} settings
   *   The dialog options.
   * @return {HTMLElement} dialog
   *   The dialog element.
   */
  Drupal.dialogNative = function dialogNative(element, settings) {
    return createDialog(element, settings);
  };
})(Drupal);
